import { TempstreamJSX, Templatable } from "tempstream";
import { BaseContext } from "koa";
import { StatefulPage } from "@sealcode/sealgen";
import html from "../html.js";
import { call_service, get_state, HA_RESPONSE } from "../ha_api.js";

export const actionName = "Dashboard";

//{"type":"call_service","domain":"automation","service":"trigger","service_data":{"entity_id":"automation.force_heating_at_20c_in_strych_for_1h","skip_condition":true},"id":41}

const actions = {
	activateStrychHeating: (state: State, inputs: Record<string, string>) => {
		call_service(
			"automation",
			"trigger",
			"automation.force_heating_at_20c_in_strych_for_1h"
		);
		return state;
	},
	remove: (state: State, _: unknown, index_to_remove: number) => {
		return {
			...state,
			elements: state.elements.filter((_, index) => index != index_to_remove),
		};
	},
} as const;

type State = {
	elements: string[];
};

async function with_unit(s: HA_RESPONSE | Promise<HA_RESPONSE>) {
	s = await s;
	return `${s.state}${s.attributes.unit_of_measurement as string}`;
}

async function arrow(entity: string) {
	const number = parseFloat((await get_state(entity)).state);
	return number < 0 ? "↓" : number > 0 ? "↑" : "=";
}

export default new (class DashboardPage extends StatefulPage<State, typeof actions> {
	actions = actions;

	getInitialState() {
		return { elements: ["one", "two", "three"] };
	}

	wrapInLayout(ctx: BaseContext, content: Templatable): Templatable {
		if (ctx.headers["just-data"]) {
			return content;
		} else {
			return html(ctx, "Dashboard", content, { navbar: () => "" });
		}
	}

	async render(ctx: BaseContext, state: State, inputs: Record<string, string>) {
		const date = new Date();

		return (
			<div style="font-size: 2rem">
				<div style="font-size: 10rem; text-align: center;">
					{date.getHours().toString().padStart(2, "0")}:
					{date.getMinutes().toString().padStart(2, "0")}
				</div>
				<table style="margin: 0 auto">
					<tbody>
						<tr>
							<td style="text-align: right">Temperatura na balkonie:</td>
							<td>
								<strong>
									{with_unit(get_state("input_number.balkon_average"))}
									{arrow("sensor.balkon_temperature_derivative")}
								</strong>
							</td>
						</tr>
						<tr>
							<td style="text-align: right">Temperatura w salonie:</td>
							<td>
								<strong>
									{with_unit(
										get_state("sensor.ewelink_th01_temperature")
									)}
									{arrow("sensor.temperatura_w_salonie_derivative")}
								</strong>
							</td>
						</tr>
						<tr>
							<td style="text-align: right">Wilgotność w salonie</td>
							<td>
								<strong>
									{with_unit(get_state("sensor.ewelink_th01_humidity"))}
								</strong>
							</td>
						</tr>
						<tr>
							<td style="text-align: right">Temperatura na strychu</td>
							<td>
								<strong>
									{with_unit(get_state("sensor.strych_temperature_3"))}
									{arrow("sensor.strych_temperature_derivative")}
								</strong>
							</td>
						</tr>
						{/*
						<tr>
							<td style="text-align: right">Docelowa temp. na strychu</td>
							<td>
								<strong>
									{with_unit(
										get_state(
											"input_number.strych_target_temperature"
										)
									)}
								</strong>
							</td>
							</tr>*/}
					</tbody>
				</table>
				{this.makeActionButton(state, {
					action: "activateStrychHeating",
					label: "Włącz-ogrzewanie-na-strychu-na-1h",
				})}
			</div>
		);
	}
})();
